﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// .NET Source Code
// Copyright © 2012 MobilEyze, LLC
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
// 
// Author	Date		Comment
// =========================================================================================================
// NLW      12 Feb 2012 Created 
/////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Windows.Controls;
using System.Windows;
using System.ComponentModel;

namespace MobilEyze.Tools.DataLogger.ToolSuite.Utilities
{
    /// <summary>
    /// A DataGrid Implementation that limits the number of rows.
    /// </summary>
    public class MaxRowsDataGrid : DataGrid
    {
        public static readonly DependencyProperty MaxRowsProperty =
            DependencyProperty.Register("MaxRows", typeof(int), typeof(MaxRowsDataGrid),
            new PropertyMetadata(0, MaxRowsPropertyChanged)
            );

        private static void MaxRowsPropertyChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            MaxRowsDataGrid maxdg = sender as MaxRowsDataGrid;
            maxdg.SetCanUserAddRowsState();   
        }

        public int MaxRows
        {
            get { return (int)GetValue(MaxRowsProperty); }
            set { SetValue(MaxRowsProperty, value); }
        }

        private volatile bool _ChangingState = false;
        private void SetCanUserAddRowsState()
        {
            if (_ChangingState == false)
            {
                _ChangingState = true;
                int maxRows = (CanUserAddRows == true) ? this.MaxRows : this.MaxRows - 1;
                if (this.Items.Count > maxRows)
                {
                    this.CanUserAddRows = false;
                }
                else
                {
                    this.CanUserAddRows = true;
                }
                _ChangingState = false;
            }
        }

        private bool IsInEditMode()
        {
            IEditableCollectionView itemsView = Items;
            if (itemsView.IsAddingNew == false && itemsView.IsEditingItem == false)
            {
                return false;
            }

            return true;
        }

        
        protected override void OnItemsChanged(System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            base.OnItemsChanged(e);
            if (this.IsInEditMode() == true)
            {
                EventHandler eh = null;
                eh += new EventHandler(delegate
                     {
                         if (IsInEditMode() == false)
                         {
                             SetCanUserAddRowsState();
                             LayoutUpdated -= eh;
                         }
                     });
                LayoutUpdated += eh;
            }
            else
            {
                SetCanUserAddRowsState();
            }

        }
    }
}
