﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace MobilEyze.Tools.DataLogger.ToolSuite.View_Models
{
    /// <summary>
    /// Simple object that can be used for tracing application messages.
    /// </summary>
    public class AppTraceVM
    {

        #region Singleton

        private static volatile AppTraceVM _Instance = null;
        private static volatile object _Lock = new object();

        public static AppTraceVM GetInstance()
        {
            lock (_Lock)
            {
                if (_Instance == null)
                {
                    _Instance = new AppTraceVM();
                }

                return _Instance;
            }
        }

        #endregion

        #region Constructor

        public AppTraceVM()
        {
            this.Messages = new ObservableCollection<AppTraceMessage>();
            this.InformationMessages = new ObservableCollection<AppTraceMessage>();
            this.WarningMessages = new ObservableCollection<AppTraceMessage>();
            this.CriticalMessages = new ObservableCollection<AppTraceMessage>();
        }


        #endregion

        #region Messages

        public void AddMessage(string source, string message, E_AppTraceSeverity severity)
        {
            AppTraceMessage msg = new AppTraceMessage();
            msg.Source = source;
            msg.Message = message;
            msg.Severity = severity;

            this.Messages.Add(msg);

            switch (severity)
            {
                case E_AppTraceSeverity.Information:
                    this.InformationMessages.Add(msg);
                    break;

                case E_AppTraceSeverity.Warning:
                    this.WarningMessages.Add(msg);
                    break;

                case E_AppTraceSeverity.Critical:
                    this.CriticalMessages.Add(msg);
                    break;
            }

        }

        public ObservableCollection<AppTraceMessage> Messages { get; private set; }
        public ObservableCollection<AppTraceMessage> InformationMessages { get; private set; }
        public ObservableCollection<AppTraceMessage> WarningMessages { get; set; }
        public ObservableCollection<AppTraceMessage> CriticalMessages { get; set; }

        #endregion
    }

    #region App Trace Message Class
    public enum E_AppTraceSeverity
    {
        Information,
        Warning,
        Critical,
    }

    public class AppTraceMessage
    {
        public AppTraceMessage()
        {
            this.Date = DateTime.Now;
        }

        public string Source { get; set; }
        public string Message { get; set; }
        public DateTime Date { get; set; }
        public E_AppTraceSeverity Severity { get; set; }
    }
    #endregion
}
