﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using FTFramework.WPF.Utilities;
using MobilEyze.Tools.Firmware.FirmwareBin;
using MobilEyze.Tools.Firmware;
using System.IO;
using System.Diagnostics;
using System.ComponentModel;

namespace MobilEyze.Tools.DataLogger.ToolSuite.View_Models
{
    public class BuildFirmwareVM : INotifyPropertyChanged
    {

        #region Public Properties

        public string SourceFileName { get; set; }
        public E_ProgrammingSegmentBlockSize ProgrammingBlockSize { get; set; }
        public string Message { get; set; }

        #endregion

        #region Build Firmware Command
        public ICommand BuildFirmware { get { return new RelayCommand(parm => this.OnBuildFirmware(), parm => this.OnOkToBuildFirmware()); } }

        public void OnBuildFirmware()
        {
            SetUserMessage("Preparing to build...");

            // Retrieve the version from the virtual sr file
            VirtualSR vsr = new VirtualSR();
            vsr.LoadSRFile(this.SourceFileName,"Need to implement this parm!");
            uint iVersion = vsr.GetAITVersion();

            string BinFileName = Path.GetFileNameWithoutExtension(this.SourceFileName) + ".bin";

            switch (iVersion)
            {
                case 0:
                    try
                    {
                        FirmwareBin_Version_0.CreateBinFile(BinFileName, vsr);
                        SetUserMessage("Firmware file built successfully.");
                    }
                    catch (Exception ex)
                    {
                        SetUserMessage(string.Format("Ex: {0}", ex.Message));
                    }
                    break;

                default:
                    Debug.WriteLine("Source File contains an unsupported AIT Version({0})", iVersion);
                    SetUserMessage(String.Format("Source File contains an unsupported AIT Version({0})", iVersion));
                    return;
            }
        }

        public bool OnOkToBuildFirmware()
        {
            return !string.IsNullOrEmpty(this.SourceFileName);
        }

        #endregion

        #region Helper Functions
        private void SetUserMessage(string msg)
        {
            this.Message = msg;
            FirePropertyChangedEvent("Message");
        }
        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChangedEvent(string PropertyName)
        {
            PropertyChangedEventHandler eh = PropertyChanged;
            if (eh != null)
                eh(this, new PropertyChangedEventArgs(PropertyName));
        }
        #endregion
    }
}
