﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// .NET Source Code
// Copyright © 2012 MobilEyze, LLC
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
// 
// Author	Date		Comment
// =========================================================================================================
// NLW      12 Feb 2012 Created 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using MobilEyze.Tools.DataLogger.ToolSuite.Model;
using System.Collections.ObjectModel;
using MobilEyze.Tools.DataLogger.ToolSuite.Utilities;
using System.Windows.Input;
using System.Windows;
using System.Data;
using System.ComponentModel;
using System.Text;
using System.Windows.Controls;
using System.Globalization;
using System.Windows.Data;

namespace MobilEyze.Tools.DataLogger.ToolSuite.View_Models
{
    /// <summary>
    /// Configuration File View Model which serves as the bridge between the UI and the actual configuration file.
    /// </summary>
    public class ConfigurationFileVM : WorkspaceVM
    {
        #region CAN Filter
        /// <summary>
        /// Class that implements the IDataErrorInfo making it great for binding with DataGrid's
        /// </summary>
        public class CANFilter : IDataErrorInfo
        {
            #region Event
            public event EventHandler CANFilterChanged;
            private void FireCANFilterChangedEvent()
            {
                EventHandler eh = CANFilterChanged;
                if (eh != null)
                {
                    eh(this, null);
                }
            }
            #endregion

            #region Constructors

            public CANFilter() 
            {
                this.MessageType = MEConfigurationFile.E_CAN_MSG_TYPE.Standard;
                this.StartingMessageID = 0x0;
                this.EndingMessageID = MEConfigurationFile.MAX_11BIT_ID;
                
            }

            public CANFilter(MEConfigurationFile.CANFilter canFilter)
            {
                try
                {
                    this.MessageType = canFilter.MessageType;
                    this.StartingMessageID = canFilter.StartingMessageID;
                    this.EndingMessageID = canFilter.EndingMessageID;
                    
                }
                catch { }
            }
            #endregion
            
            #region Public Properties

            private uint _StartingMessageID = 0;
            public uint StartingMessageID 
            { 
                get { return _StartingMessageID; } 
                
                set
                {
                    ValidateMsgID(this.MessageType, value);
                    _StartingMessageID = value;
                    FireCANFilterChangedEvent();

                }
            }


            private uint _EndingMessageID = 0;
            public uint EndingMessageID 
            {
                get { return _EndingMessageID; }

                set
                {
                    ValidateMsgID(this.MessageType, value);
                    _EndingMessageID = value;
                    FireCANFilterChangedEvent();
                }
            }

            MEConfigurationFile.E_CAN_MSG_TYPE _MessageType;
            public MEConfigurationFile.E_CAN_MSG_TYPE MessageType 
            {
                get { return _MessageType; }
                set
                {
                    _MessageType = value;
                    ValidateMsgID(_MessageType, this.StartingMessageID);
                    ValidateMsgID(_MessageType, this.EndingMessageID);
                    FireCANFilterChangedEvent();
                }
            }



            #endregion
            
            #region Helpers
            private void ValidateMsgID(MEConfigurationFile.E_CAN_MSG_TYPE msgType, string msgID)
            {
                ValidateMsgID(msgType, System.Convert.ToUInt32(msgID, 16));
            }

            private void ValidateMsgID(MEConfigurationFile.E_CAN_MSG_TYPE msgType, uint msgID)
            {
                if (msgType == MEConfigurationFile.E_CAN_MSG_TYPE.Standard)
                {
                    if (msgID > MEConfigurationFile.MAX_11BIT_ID)
                    {
                        // The DataGrid catches these exceptions and displays them for us.
                        throw new ArgumentException(string.Format("11 Bit message id's can't be greater than 0x{0:X3}", MEConfigurationFile.MAX_11BIT_ID));
                    }
                }
                else
                {
                    if (msgID > MEConfigurationFile.MAX_29BIT_ID)
                    {
                        // The DataGrid catches these exceptions and displays them for us.
                        throw new ArgumentException(string.Format("29 Bit message id's can't be greater than 0x{0X8}", MEConfigurationFile.MAX_29BIT_ID));
                    }
                }
            }
            #endregion

            #region IDataErrorInfo Members

            /// <summary>
            /// A string representing any row level exceptions - starting id > ending id for instance.
            /// </summary>
            public string Error
            {
                get
                {
                    StringBuilder error = new StringBuilder();

                    // iterate over all of the properties
                    // of this object - aggregating any validation errors
                    PropertyDescriptorCollection props = TypeDescriptor.GetProperties(this);
                    foreach (PropertyDescriptor prop in props)
                    {
                        string propertyError = this[prop.Name];
                        if (propertyError != string.Empty)
                        {
                            error.Append((error.Length != 0 ? ", " : "") + propertyError);
                        }
                    }

                    // apply object level validation rules
                    if (StartingMessageID > EndingMessageID)
                    {
                        error.Append((error.Length != 0 ? ", " : "") + "The ending message id must be greater than or equal to the starting message id.");
                    }

                    if (MessageType == MEConfigurationFile.E_CAN_MSG_TYPE.Standard)
                    {
                        if (StartingMessageID > MEConfigurationFile.MAX_11BIT_ID || EndingMessageID > MEConfigurationFile.MAX_11BIT_ID)
                        {
                            error.Append((error.Length != 0 ? ", " : "") + string.Format("Message id's aren't within the valid range for 11Bit ids (0x0-0x{0:X3}.", MEConfigurationFile.MAX_11BIT_ID));
                        }
                    }
                    else
                    {
                        if (StartingMessageID > MEConfigurationFile.MAX_29BIT_ID || EndingMessageID > MEConfigurationFile.MAX_29BIT_ID)
                        {
                            error.Append((error.Length != 0 ? ", " : "") + string.Format("Message id's aren't within the valid range for 11Bit ids (0x0-0x{0:X8}.",MEConfigurationFile.MAX_29BIT_ID));
                        }
                    }

                    return error.ToString();
                }
            }

            /// <summary>
            /// Can handle column level validation here if desired.
            /// </summary>
            /// <param name="columnName"></param>
            /// <returns></returns>
            public string this[string columnName]
            {
                get 
                {
                    // Add Column level checking here if desired.
                    //switch (columnName)
                    //{

                    //}

                    return string.Empty;
                
                }
            }

            #endregion
        }
        #endregion

        #region Constructor

        public ConfigurationFileVM()
        {
            this.CANFilters = new ObservableCollection<CANFilter>();
        }

        #endregion

        #region Private Properties

        private MEConfigurationFile ConfigFile { get;  set; }

        #endregion

        #region Public Properties

        public List<string> SupportedBusSpeeds
        {
            get
            {
                return MEConfigurationFile.GetSupportedBusSpeeds();
            }
        }

        // We need to keep track of changes so that we can update the Display name and Enable/Disable the Save button
        string _SelectedBusSpeed = string.Empty;
        public string SelectedBusSpeed 
        {
            get { return _SelectedBusSpeed; }
            set
            {
                if (_SelectedBusSpeed != value)
                {
                    _SelectedBusSpeed = value;
                    this.SetDirty(true);
                }
            }

        }

        public List<string> SupportedLogTypes
        {
            get
            {
                return MEConfigurationFile.GetSupportedLogTypes();
            }
        }

        string _SelectedLogType = string.Empty;
        public string SelectedLogType 
        {
            get { return _SelectedLogType; }
            set
            {
                if (value != _SelectedLogType)
                {
                    _SelectedLogType = value;
                    this.SetDirty(true);
                }
            }
        }

        bool _OutputDebugToCAN = false;
        public bool OutputDebugToCAN 
        {
            get { return _OutputDebugToCAN; }
            set
            {
                if (_OutputDebugToCAN != value)
                {
                    _OutputDebugToCAN = value;
                    SetDirty(true);
                }
            }
        }


        bool _UseErrorPassiveDetection = false;
        public bool UseErrorPassiveDetection 
        {
            get { return _UseErrorPassiveDetection; }
            set
            {
                if (_UseErrorPassiveDetection != value)
                {
                    _UseErrorPassiveDetection = value;
                    SetDirty(true);
                }
            }
        }

        uint _CANId = 0xFE;
        public uint CANId 
        {
            get { return _CANId; }
            set
            {
                if (_CANId != value)
                {
                    _CANId = value;
                    SetDirty(true);
                }
            }
        }

        public ObservableCollection<CANFilter> CANFilters { get; set; }

        public int MaxCANFilters { get { return 10; } }

        #endregion

        #region WorkspaceVM implementations

        #region Create New
        /// <summary>
        /// Create a new configuration file workspace using the file name provided.
        /// </summary>
        /// <param name="path"></param>
        public void CreateNewConfigFile(string path)
        {
            this.ConfigFile = MEConfigurationFile.Create(path);
            this.Initialize();
            this.SetDirty(true);
        }

        #endregion

        #region Open
        /// <summary>
        /// Load an existing configuration file into this VM.
        /// </summary>
        /// <param name="path"></param>
        public void LoadConfigurationFile(string path)
        {
            this.ConfigFile = MEConfigurationFile.Load(path);
            this.Initialize();
            this.SetDirty(false);
        }

        #endregion

        #region Initialize

        /// <summary>
        /// Initialize the VM
        /// </summary>
        private void Initialize()
        {
            this.SelectedBusSpeed = MEConfigurationFile.GetBusSpeedString(this.ConfigFile.BusSpeed);
            this.SelectedLogType = MEConfigurationFile.GetLogTypeString(this.ConfigFile.LogType);

            this.OutputDebugToCAN = this.ConfigFile.OutputDebugToCAN;
            this.CANFilters = new ObservableCollection<CANFilter>();
            foreach (MEConfigurationFile.CANFilter filter in this.ConfigFile.Filters)
            {
                if (filter != null)
                {
                    CANFilter cf = new CANFilter(filter);
                    this.CANFilters.Add(cf);
                    cf.CANFilterChanged += new EventHandler(cf_CANFilterChanged);
                }
            }
            this.CANFilters.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(CANFilters_CollectionChanged);
            this.UseErrorPassiveDetection = this.ConfigFile.UseErrorPassiveDetection;
            this.CANId = this.ConfigFile.CANId;
        }

        void cf_CANFilterChanged(object sender, EventArgs e)
        {
            SetDirty(true);
        }

        void CANFilters_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            SetDirty(true);
        }

        #endregion

        #region Set Display Name
        /// <summary>
        /// Sets the display name based on the file name and the state of the VM.
        /// </summary>
        public void SetDisplayName()
        {
            string fileName = System.IO.Path.GetFileName(this.ConfigFile.FileName);
            this.DisplayName = string.Format("{0}{1}", (this.IsDirty ? "*" : ""),  fileName);
            FirePropertyChangedEvent("DisplayName");

        }
        #endregion

        #region Set Dirty Flag and Update related properties
        private void SetDirty(bool isDirty)
        {
            this.IsDirty = isDirty;
            SetDisplayName();
        }

        #endregion

        #region Closing
        /// <summary>
        /// Override the OnClosing if special work needs to be performed.
        /// </summary>
        protected override void OnClosing()
        {
            // Nothing for us to do.
        }

        #endregion

        #region Saving
        /// <summary>
        /// Override the OnSaving method so that we can implement the details for saving the config file.
        /// </summary>
        /// <returns></returns>
        protected override bool OnSaving()
        {
            // Save the config file
            this.ConfigFile.SetBusSpeed(this.SelectedBusSpeed);
            this.ConfigFile.SetLogType(this.SelectedLogType);
            this.ConfigFile.OutputDebugToCAN = this.OutputDebugToCAN;
            this.ConfigFile.UseErrorPassiveDetection = this.UseErrorPassiveDetection;
            this.ConfigFile.CANId = this.CANId;

            // Set the filters
            this.ConfigFile.ClearFilters();
            foreach (CANFilter filter in this.CANFilters)
            {
                this.ConfigFile.AddFilter(filter.StartingMessageID, filter.EndingMessageID, filter.MessageType);
            }
            MEConfigurationFile.Save(this.ConfigFile);
            SetDirty(false);
            return true;
        }
        #endregion

        #region Supported Message Types - For binding in the Data Grid Combo Box
        public class SupportedMessageType
        {
            public string DisplayValue { get; set; }
            public MEConfigurationFile.E_CAN_MSG_TYPE MessageType { get; set; }
            public override string ToString()
            {
                return string.Format("{0} : {1}", this.DisplayValue, this.MessageType.ToString());
            }
        }

        ObservableCollection<SupportedMessageType> _SupportedMessageTypes = null;
        public ObservableCollection<SupportedMessageType> SupportedMessageTypes
        {
            get
            {
                if (_SupportedMessageTypes == null)
                {
                    _SupportedMessageTypes = new ObservableCollection<SupportedMessageType>();
                    _SupportedMessageTypes.Add(new SupportedMessageType() { DisplayValue = "11 Bit Message", MessageType = MEConfigurationFile.E_CAN_MSG_TYPE.Standard });
                    _SupportedMessageTypes.Add(new SupportedMessageType() { DisplayValue = "29 Bit Message", MessageType = MEConfigurationFile.E_CAN_MSG_TYPE.Extended });
                }

                return _SupportedMessageTypes;
            }

        }
        
        
        #endregion

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Cleans up the VM
        /// </summary>
        public override void Dispose()
        {
            if (this.CANFilters != null)
            {
                foreach (CANFilter cf in this.CANFilters)
                {
                    cf.CANFilterChanged -= new EventHandler(cf_CANFilterChanged);
                }
            }
        }

        #endregion
    }

    #region Validation Rules 
    /// <summary>
    /// The main class for handling row level validation for the CAN Filters.
    /// </summary>
    public class RowDataInfoValidationRule : ValidationRule
    {
        public override ValidationResult Validate(object value, CultureInfo cultureInfo)
        {
            BindingGroup group = (BindingGroup)value;

            StringBuilder error = null;
            // Loop through the bound colums to determine if there are any validation error.
            foreach (var item in group.Items)
            {
                // aggregate errors
                IDataErrorInfo info = item as IDataErrorInfo;
                if (info != null)
                {
                    if (!string.IsNullOrEmpty(info.Error))
                    {
                        if (error == null)
                            error = new StringBuilder();
                        error.Append((error.Length != 0 ? ", " : "") + info.Error);
                    }
                }
            }

            if (error != null)
                return new ValidationResult(false, error.ToString());

            return ValidationResult.ValidResult;
        }
    }

    /// <summary>
    /// Just a Dummy Rule.
    /// </summary>
    public class DummyValidationRule : ValidationRule
    {

        public override ValidationResult Validate(object value, System.Globalization.CultureInfo cultureInfo)
        {
            return ValidationResult.ValidResult;
        }
    }

    #endregion

}
