﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// .NET Source Code
// Copyright © 2012 MobilEyze, LLC
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
// 
// Author	Date		Comment
// =========================================================================================================
// NLW      12 Feb 2012 Created 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MobilEyze.Tools.DataLogger.ToolSuite.View_Models.SetDateTime
{
    public class ModuleDateTimeChangedEventArgs : EventArgs
    {
        public DateTime CurrentDateTime { get; private set; }
        public ModuleDateTimeChangedEventArgs(DateTime dt)
        {
            this.CurrentDateTime = dt;
        }

        public ModuleDateTimeChangedEventArgs(string dt)
        {
            DateTime d_DT;
            DateTime.TryParse(dt, out d_DT);
            this.CurrentDateTime = d_DT;
        }
    }

    /// <summary>
    /// Set the modules date and time interface.
    /// </summary>
    interface ISDT
    {
        event EventHandler<ModuleDateTimeChangedEventArgs> ModuleDateTimeChanged;
        List<string> AvailablePortNames { get; }
        void Open(string portName);
        void Close();

        /// <summary>
        /// The module use epoch time.  See http://www.epochconverter.com/ for details.
        /// What is epoch time?
        /// The Unix epoch (or Unix time or POSIX time or Unix timestamp) is the number of seconds that have elapsed 
        /// since January 1, 1970 (midnight UTC/GMT), not counting leap seconds 
        /// (in ISO 8601: 1970-01-01T00:00:00Z). Literally speaking the epoch is Unix time 0 (midnight 1/1/1970), 
        /// but 'epoch' is often used as a synonym for 'Unix time'. Many Unix systems store epoch dates as a signed 32-bit integer, 
        /// which might cause problems on January 19, 2038 (known as the Year 2038 problem or Y2038).  
        /// </summary>
        void SetModuleDateTimeCurrent();
        void SetModuleDateTimeGMT();
    }
}
