﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// .NET Source Code
// Copyright © 2012 MobilEyze, LLC
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
// 
// Author	Date		Comment
// =========================================================================================================
// NLW      12 Feb 2012 Created 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MobilEyze.Tools.DataLogger.ToolSuite.View_Models.SetDateTime
{
    /// <summary>
    /// Base class for classes that set the modules date and time.
    /// </summary>
    public abstract class SDTBase : ISDT
    {
        #region Events

        /// <summary>
        /// Fire when the implementing class determines that the date and time has changed.
        /// </summary>
        /// <param name="dt">Must be a valid DateTime value</param>
        protected void FireModuleDateTimeChangedEvent(DateTime dt)
        {
            EventHandler<ModuleDateTimeChangedEventArgs> eh = ModuleDateTimeChanged;
            if (eh != null)
            {
                eh(this, new ModuleDateTimeChangedEventArgs (dt));
            }
        }

        /// <summary>
        /// Fire when the implementing class determines that the date and time has changed.
        /// </summary>
        /// <param name="dt">a string value that can be converted to a valid date value</param>
        protected void FireModuleDateTimeChangedEvent(string dt)
        {
            EventHandler<ModuleDateTimeChangedEventArgs> eh = ModuleDateTimeChanged;
            if (eh != null)
            {
                eh(this, new ModuleDateTimeChangedEventArgs(dt));
            }
        }

        #endregion

        #region Constructor

        public SDTBase()
        {
            this.AvailablePortNames = new List<string>();
        }

        #endregion

        #region ISDT Members

        /// <summary>
        /// Event is fired when the implementing class determines that the modules date and time has changed.
        /// </summary>
        public event EventHandler<ModuleDateTimeChangedEventArgs> ModuleDateTimeChanged;

        /// <summary>
        /// A list of available ports that can be used by the implementing class.
        /// </summary>
        public List<string> AvailablePortNames { get; protected set; }

        public abstract void Open(string portName);
        public abstract void Close();

        /// <summary>
        /// The module use epoch time.  See http://www.epochconverter.com/ for details.
        /// What is epoch time?
        /// The Unix epoch (or Unix time or POSIX time or Unix timestamp) is the number of seconds that have elapsed 
        /// since January 1, 1970 (midnight UTC/GMT), not counting leap seconds 
        /// (in ISO 8601: 1970-01-01T00:00:00Z). Literally speaking the epoch is Unix time 0 (midnight 1/1/1970), 
        /// but 'epoch' is often used as a synonym for 'Unix time'. Many Unix systems store epoch dates as a signed 32-bit integer, 
        /// which might cause problems on January 19, 2038 (known as the Year 2038 problem or Y2038).  
        /// </summary>
        public abstract void SetModuleDateTimeCurrent();
        public abstract void SetModuleDateTimeGMT();
        #endregion
    }
}
