﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// .NET Source Code
// Copyright © 2012 MobilEyze, LLC
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
// 
// Author	Date		Comment
// =========================================================================================================
// NLW      12 Feb 2012 Created 
/////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using MobilEyze.Tools.DataLogger.ToolSuite.Utilities;

namespace MobilEyze.Tools.DataLogger.ToolSuite.View_Models
{
    /// <summary>
    /// The base worksapce view model.
    /// </summary>
    public abstract class WorkspaceVM : ViewModelBase, IDisposable
    {

        #region Constructor
        public WorkspaceVM()
        {
            this.IsDirty = false;
        }
        #endregion

        #region Save Logic
        
        /// <summary>
        /// The save command implementation.
        /// </summary>
        public ICommand SaveCommand { get { return new RelayCommand(parm => this.OnSave(), parm=>this.OnOkToSave()); } }

        /// <summary>
        /// Handle saving the worksapce
        /// </summary>
        private void OnSave()
        {

            // Call the abstract method
            if (OnSaving())
            {
                this.IsDirty = false;
            }
        }

        /// <summary>
        /// Abstract method - called when the workspace needs to be saved.
        /// </summary>
        protected abstract bool OnSaving();

        /// <summary>
        /// Indicates when it's ok to save the workspace.
        /// </summary>
        /// <returns></returns>
        protected virtual bool OnOkToSave()
        {
            return this.IsDirty;
        }

        #endregion

        #region Close logic
        /// <summary>
        /// The workspace fires this event when it want to close.
        /// </summary>
        public event EventHandler RequestClose;

        /// <summary>
        /// The close command implementation
        /// </summary>
        public ICommand CloseCommand { get { return new RelayCommand(parm => this.OnClose(), parm => this.OnOkToClose()); } }
        private void OnClose()
        {
            // Call the abstract method - give implementor a chance to respond
            OnClosing();

            // Fire the Request Close Command
            EventHandler eh = RequestClose;
            if (eh != null)
            {
                eh(this, null);
            }
        }

        /// <summary>
        /// Abstract method - called when the workspace needs to be closed.
        /// </summary>
        protected abstract void OnClosing();

        private bool OnOkToClose()
        {
            // If the workspace is dirty then we return false, otherwise we return true
            return !this.IsDirty;
        }

        #endregion

        #region Dirty logic
        public bool IsDirty { get; set; }

        #endregion


        #region IDisposable Members

        public virtual void Dispose()
        {
            
        }

        #endregion
    }
}
